use super::{archive, package, RepoHandle};
use crate::db;

use std::{
    collections::HashMap,
    path::{Path, PathBuf},
    sync::{atomic::AtomicU32, Arc, Mutex, RwLock},
};

use sea_orm::{
    ActiveModelTrait, ColumnTrait, Condition, ConnectionTrait, DbConn, EntityTrait, JoinType,
    ModelTrait, NotSet, QueryFilter, QuerySelect, Related, RelationTrait, Set, TransactionTrait,
};
use tokio::{runtime, sync::mpsc::UnboundedReceiver};

pub enum RepoCommand {
    ParsePkg(i32, PathBuf),
}

/// The actor is responsible for mutating the repositories. They receive their commands their
/// messages and process these commands in both a synchronous and asynchronous way.
pub struct RepoActor {
    repos_dir: PathBuf,
    conn: DbConn,
    rt: runtime::Handle,
    rx: Arc<Mutex<UnboundedReceiver<RepoCommand>>>,
    repos: Arc<RwLock<HashMap<i32, (AtomicU32, Arc<Mutex<()>>)>>>,
}

impl RepoActor {
    pub fn new(
        repos_dir: PathBuf,
        conn: DbConn,
        rt: runtime::Handle,
        rx: Arc<Mutex<UnboundedReceiver<RepoCommand>>>,
        repos: Arc<RwLock<HashMap<i32, (AtomicU32, Arc<Mutex<()>>)>>>,
    ) -> Self {
        Self {
            repos_dir,
            conn,
            rt,
            rx,
            repos,
        }
    }

    /// Run the main actor loop
    pub fn run(self) {
        while let Some(msg) = {
            let mut rx = self.rx.lock().unwrap();
            rx.blocking_recv()
        } {
            match msg {
                RepoCommand::ParsePkg(repo, path) => {
                    let _ = self.parse_pkg(repo, path);
                }
            }
        }
    }

    fn parse_pkg(&self, repo: i32, path: PathBuf) -> crate::Result<()> {
        let pkg = package::Package::open(&path)?;
        let pkg = self
            .rt
            .block_on(db::query::package::insert(&self.conn, repo, pkg))?;
        let dest_path = self
            .repos_dir
            .join(repo.to_string())
            .join(pkg.id.to_string());
        std::fs::rename(path, dest_path)?;

        tracing::info!(
            "Added '{}-{}-{}' to repository {}",
            pkg.name,
            pkg.version,
            pkg.arch,
            repo,
        );

        Ok(())
    }
}
