use crate::db;

use std::{
    collections::HashMap,
    path::{Path, PathBuf},
    sync::{atomic::AtomicU32, Arc, Mutex, RwLock},
};

use sea_orm::{
    ActiveModelTrait, ColumnTrait, Condition, ConnectionTrait, DbConn, EntityTrait, JoinType,
    ModelTrait, NotSet, QueryFilter, QuerySelect, Related, RelationTrait, Set, TransactionTrait,
};
use tokio::{
    runtime,
    sync::mpsc::{unbounded_channel, UnboundedSender},
};

#[derive(Clone)]
pub struct RepoHandle {
    repos_dir: PathBuf,
    conn: DbConn,
    tx: UnboundedSender<super::RepoCommand>,
    repos: Arc<RwLock<HashMap<i32, (AtomicU32, Arc<Mutex<()>>)>>>,
}

impl RepoHandle {
    pub fn start(
        repos_dir: impl AsRef<Path>,
        conn: DbConn,
        actors: u32,
        rt: runtime::Handle,
    ) -> crate::Result<Self> {
        std::fs::create_dir_all(repos_dir.as_ref())?;

        let (tx, rx) = unbounded_channel();

        let mut repos = HashMap::new();
        let repo_ids: Vec<i32> = rt.block_on(
            db::Repo::find()
                .select_only()
                .column(db::repo::Column::Id)
                .into_tuple()
                .all(&conn),
        )?;

        for id in repo_ids {
            repos.insert(id, Default::default());
        }

        let rx = Arc::new(Mutex::new(rx));
        let repos = Arc::new(RwLock::new(repos));

        for _ in 0..actors {
            let actor = super::RepoActor::new(
                repos_dir.as_ref().to_path_buf(),
                conn.clone(),
                rt.clone(),
                Arc::clone(&rx),
                Arc::clone(&repos),
            );

            std::thread::spawn(|| actor.run());
        }

        Ok(Self {
            repos_dir: repos_dir.as_ref().to_path_buf(),
            conn,
            tx,
            repos,
        })
    }
}
